from mipscripts import utils
import csv
import os


# TODO - this should be changed to write_fastq_from_mergedsheet
def merge_sampleset_write_fastq(
    mergedsheet, newfastqdir, skipfastqwrite, skipbadfastqs
):
    """Writes FASTQ files to a directory from a merged sample sheet.

    The merged sample sheet can be created using the `merge_samplesheet` 
    function. This function allows for more complicated merges of data that may
    require intermediate manipulation of the merged sample sheet file. For
    instance, controls may be deleted or sample sets renamed. After manipulation
    of the merged sample sheet, you may run this function to combine the FASTQ
    files into one directory.

    Args:
        mergedsheet (str): A path to the merged sample sheet.
        newfastqdir (str): The path of the new FASTQ directory where files will 
            be saved.
        skipfastqwrite (bool): A boolean indicating whether to skip the writing
            of FASTQ files. This is equivalent to a dry run.
        skipbadfastqs (bool): A boolean indicating whether to skip bad FASTQs
            without throwing errors.
    """
    print(f"CREATING NEW FASTQ DIRECTORY: `{newfastqdir}`")
    utils.make_dir(newfastqdir)
    # total_bad = 0
    # total = good = 0

    with open(mergedsheet, newline="") as items_file:
        items_reader = csv.DictReader(items_file, delimiter="\t")
        # items_total = 0
        for item in items_reader:
            fastqs = item["fastq"].split(",")
            name = "{}-{}-{}".format(
                item["sample_name"], item["sample_set"], item["replicate"]
            )
            if not skipfastqwrite:
                print("...", name, "...")
            writeoperator = " > "
            if len(fastqs) % 2 == 0 and len(fastqs) > 1:
                for i in range(0, len(fastqs), 2):
                    if not skipfastqwrite:
                        os.system(
                            "cat  "
                            + fastqs[i]
                            + writeoperator
                            + newfastqdir
                            + "/"
                            + name
                            + "_R1_001.fastq.gz"
                        )
                        os.system(
                            "cat  "
                            + fastqs[i + 1]
                            + writeoperator
                            + newfastqdir
                            + "/"
                            + name
                            + "_R2_001.fastq.gz"
                        )
                    writeoperator = " >> "  # now appending after initial files
            else:
                if skipbadfastqs:
                    print("WARN bad fasta paths for ", name, "(", fastqs, ")")
                else:
                    print("ERROR: Bad FASTQ", name, fastqs)
                    print("\u2022 Skip bad FASTQs with '--skipbadfastqs'.")
                    exit(1)



__version__ = "v0.3.4"

import argparse
from .seqrun_stats import seqrun_stats
from .merge_sampleset_write_fastq import merge_sampleset_write_fastq
from .merge_sampleset import merge_sampleset

# Define parent argument parser and subparser
parser = argparse.ArgumentParser(
    prog="mipscripts",
    description="Utilities complementing MIPTools pipelines",
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
)
parser.add_argument(
    "--version",
    action="version",
    version="%(prog)s " + __version__,
)
subparsers = parser.add_subparsers(title="subcommands")

# Argument parser for seqrun_stats
parser_seqrun = subparsers.add_parser(
    "seqrun_stats",
    description="Compute performance statistics for a sequencing run",
    epilog="Note: currently this simply scans R1 FASTQs",
    help="compute performance statistics for a sequencing run",
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
)
parser_seqrun.add_argument(
    "-s",
    "--samplesheet",
    action="append",
    required=True,
    help="sample sheet to run on",
)
parser_seqrun.add_argument(
    "-g",
    "--maingrp",
    default="sample_set",
    help="main grouping for stats",
)
parser_seqrun.add_argument(
    "-u",
    "--subgrp",
    help="sub grouping for stats",
)
parser_seqrun.set_defaults(func=seqrun_stats)

# Argument parser for merge_sampleset_write_fastq
parser_fastq = subparsers.add_parser(
    "merge_sampleset_write_fastq",
    description="Write FASTQ files using a merged sample sheet",
    help="write FASTQ files using a merged sample sheet",
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
)
parser_fastq.add_argument(
    "-s",
    "--mergedsheet",
    required=True,
    help="merged sample sheet file paths",
)
parser_fastq.add_argument(
    "-d",
    "--newfastqdir",
    default="mergedfastq",
    help="name of the new FASTQ directory",
)
parser_fastq.add_argument(
    "-n",
    "--skipfastqwrite",
    action="store_true",
    help="skip the writing of FASTQ files. This is equivalent to a dry run.",
)
parser_fastq.add_argument(
    "-b",
    "--skipbadfastqs",
    action="store_true",
    help="skip bad FASTQs without throwing errors",
)
parser_fastq.set_defaults(func=merge_sampleset_write_fastq)

# Argument parser for merge_sampleset
parser_merge = subparsers.add_parser(
    "merge_sampleset",
    description="Flexibly merge sample sets together",
    epilog=(
        "Note: we recommend using a unique name for the outputted merged sheet"
        "based on the set/probe/and other parameters"
    ),
    formatter_class=argparse.ArgumentDefaultsHelpFormatter,
    help="flexibly merge sample sets together",
)
parser_merge.add_argument(
    "-S",
    "--set",
    action="append",
    required=True,
    help="sample set to aggregate",
)
parser_merge.add_argument(
    "-p",
    "--probe",
    action="append",
    required=True,
    help="probe sets to include in merge",
)
parser_merge.add_argument(
    "-s",
    "--sheet",
    action="append",
    help="sample sheet file paths (dir fastq must be in same dir as sheet)",
)
parser_merge.add_argument(
    "-o",
    "--mergeon",
    help="fields to merge on",
    default="sample_name-sample_set-replicate",
)
parser_merge.add_argument(
    "-m",
    "--newsheet",
    help="name of new merged sample sheet",
    default="mergedsheet.tsv",
)
parser_merge.add_argument(
    "-d",
    "--newfastqdir",
    help="name of new FASTQ directory",
    default="mergedfastq",
)
parser_merge.add_argument(
    "-e",
    "--exclude",
    action="append",
    help="exclude (samples) matching text pattern",
)
parser_merge.add_argument(
    "-a",
    "--addcolumn",
    action="append",
    help="add unexpected column name",
)
parser_merge.add_argument(
    "-n",
    "--skipfastqwrite",
    action="store_true",
    help="dry run skips FASTQ merge/write but makes tsv",
)
parser_merge.add_argument(
    "-r",
    "--renamereplicates",
    action="store_true",
    help="renumber the replicates based on order",
)
parser_merge.add_argument(
    "-c",
    "--collapse",
    action="store_true",
    help="collapse to unique values in columns (not needed for downstream miptools)",
)
parser_merge.add_argument(
    "-i",
    "--ignorereplicateredundancy",
    action="store_true",
    help="collapse multiple replicates within a samplesheet (error)",
)
parser_merge.set_defaults(func=merge_sampleset)

# Parse arguments and call subcommand
args = vars(parser.parse_args())
func = args.pop("func", None)
func(**args)



from alive_progress import alive_it
from mipscripts import utils
import collections
import csv
import gzip
import os
import os.path
import pandas as pd
import re
import statistics


def seqrun_stats(samplesheet, maingrp, subgrp):
    """Computes performance statistics for a sequencing run.

    Reads the R1 FASTQ files that correspond to a sample sheet and counts the 
    number of reads for each sample. Prints summary statistics on the number
    of reads and creates a new sample sheet file that contains the original
    data and a new column `read_count` with the number of reads per sample.

    Args:
        samplesheet (list): A list of paths to sample sheets.
        maingrp (str): The primary grouping for the data.
        subgrp (str): A secondary grouping for the data.
    """

    for sheet in samplesheet:
        # Convert header to snake case
        utils.header_to_snake_case(path=sheet, overwrite=True)

        # Initialize a counter for the number of sample sets
        num_sample_sets = collections.Counter()

        # Initialize the grouping dictionary
        groupings = {}

        # Load the sample sheet header and data into memory for fast retrieval
        # Also update the number of sample per sample set
        sampledict = []
        header = []
        with open(sheet) as samplefile:
            dictreader = csv.DictReader(samplefile, delimiter="\t")
            header = dictreader.fieldnames
            for row in dictreader:
                sampledict.append(row)
                num_sample_sets.update({row["sample_set"]: 1})

        # Error if arguments are not correct
        if maingrp not in header:
            print("ERROR: '--maingrp' invalid.")
            print(f"\u2022 User entered '{maingrp}'.")
            print(f"\u2022 Available options: {header}.")
            exit()
        if subgrp and subgrp not in header:
            print("ERROR: '--subgrp' invalid.")
            print(f"\u2022 User entered '{maingrp}'.")
            print(f"\u2022 Available options: {header}.")
            exit()

        print("####################### FASTQS / READS ########################")
        fastqdir = os.path.join(os.path.dirname(sheet), "fastq")
        fastqs = os.listdir(fastqdir)
        fqss = [f for f in fastqs if "_R1_" in f]

        # Count the number of fastq reads in each fastq file
        fastqlen = {}
        bar = alive_it(fqss)
        for fq in bar:
            reads = 0
            with gzip.open(f"{fastqdir}/{fq}", mode="rt") as f:
                for line in f:
                    if line.startswith("@"):
                        reads += 1
            fastqlen[fq] = reads
            bar.title("Counting the number of FASTQ reads")

        # Save all fastq file names
        fqfiles = fastqlen.keys()

        # For each row in our data, check if there are multiple fastqs and, if
        # not, set the read count
        for row in sampledict:
            # Define identifier to compare fastqs
            fqname = "{}-{}-{}_".format(
                row["sample_name"], row["sample_set"], row["replicate"]
            )

            # Use regex to check if the fastq name appears multiple times
            regex = re.compile("^" + fqname)
            fqmatchs = list(filter(regex.match, fqfiles))

            if len(fqmatchs) > 1:
                # Throw an error
                print(
                    "ERROR: More than one FASTQ for the sampleset repetition.",
                    fqmatchs,
                    fqname,
                )
                print(
                    "\u2022 This may be because all the fastqs were not erased before"
                )
                print(
                    "  an updated sample sheet was demultiplexed (different S###)."
                )
                exit()
            elif len(fqmatchs) == 1:
                # Set read count
                row["read_count"] = fastqlen[fqmatchs[0]]

            # Create groupings
            # TODO - we don't do anything with the grouping variables
            groupings[row[maingrp]] = {}
            if subgrp:
                groupings[row[maingrp]][row[subgrp]] = 1

        # If there were undetermined reads let the user know
        if "Undetermined_S0_R1_001.fastq.gz" in fastqlen:
            cnt = fastqlen["Undetermined_S0_R1_001.fastq.gz"]
            print(f"There were {cnt} undetermined reads from demultipelxing.")

        # Create updated sample sheet
        print("Creating an updated samplesheet with read counts.")
        sheet_root = os.path.splitext(sheet)[0]
        read_cnt_path = f"{sheet_root}_readcnt.tsv"
        with open(read_cnt_path, mode="wt") as samplesheetout:
            header.append("read_count")
            dictwriter = csv.DictWriter(
                samplesheetout, delimiter="\t", fieldnames=header
            )
            dictwriter.writeheader()
            for row in sampledict:
                dictwriter.writerow(row)

        print("########################### SAMPLES ###########################")
        print("PER SAMPLE SET SUMMARY:")
        # print(f"SAMPLE GROUPINGS AND # SAMPLES: {groupings}")

        # Print some summary stats on reads for each sample set.
        # Read the read count file in
        read_cnt_data = pd.read_csv(read_cnt_path, sep="\t")
        for key in num_sample_sets:
            # Print sample set name and number of samples
            print(f"Sample set {key} ({num_sample_sets[key]:,} samples):")

            # Filter data to sample set and isolate reads
            filter_data = read_cnt_data[read_cnt_data.sample_set == key]
            num_samples = filter_data.shape[0]
            reads = filter_data.read_count

            # Print summary stats
            print(f"... TOTAL read pairs: {sum(reads):,}")
            print(f"... MEAN read pairs: {statistics.mean(reads):,}")
            print(f"... MEDIAN read pairs: {statistics.median(reads):,}")

            # Deciles
            if num_samples > 1:
                quantiles = [
                    f"{round(q):,}" for q in statistics.quantiles(reads)
                ]
                print(f"... QUANTILES for read pairs: {quantiles}")



from collections import Counter
import csv
import os
import re
from string import punctuation, whitespace


def header_to_snake_case(path, overwrite=True):
    """Converts header column names of a file to snake case.

    Args:
        path (str): The path to the file.
        overwrite (bool): A boolean indicating whether to overwrite the original
            file. If `False`, a new file will be generated with the same name
            as the original file but with `_fix` appended to it.
    """
    with open(path) as file:
        file_content = csv.reader(file, delimiter="\t")
        header = next(file_content)
        header = [to_snake_case(col) for col in header]

        data = []
        for row in file_content:
            data.append(row)

    # Check if there are duplicate column names
    duplicate = [k for k, v in Counter(header).items() if v > 1]
    if len(duplicate) > 0:
        print("\nERROR: Header has duplicate columns.")
        print(f"\u2022 Duplicate columns: '{duplicate}'.")
        exit()

    outfile = path if overwrite else "_fix".join(os.path.splitext(path))
    with open(outfile, mode="w") as file:
        file_content = csv.writer(file, delimiter="\t")
        file_content.writerow(header)
        file_content.writerows(data)


def to_snake_case(string):
    """Converts a string to snake case

    Args:
        string (str): A string.
    """
    # Define whitespace and punctuation regex string
    space_punc = f"[{whitespace}{re.escape(punctuation)}]"
    
    # Remove underscore from search string
    space_punc = re.sub("_", "", space_punc)
    regex_space_punc = re.compile(space_punc)

    # Warn user that string is being converted to snake case
    if re.search(regex_space_punc, string) or re.search("[A-Z]", string):
        print(f"Converting `{string}` to snake case.")
    
    # Convert whitespace and punctuation to underscore
    string = re.sub(regex_space_punc, "_", string)

    # Add underscore before capitalized letters
    string = re.sub(r"(?<!^)(?=[A-Z])", "_", string)

    # Convert multiple underscores to one underscore
    string = re.sub("_+", "_", string)

    # Lowercase everything
    return string.lower()


def make_dir(dir, overwrite=True):
    """Checks if a directory can be made and creates it.

    Args:
        dir (str): Path to the directory.
        overwrite (bool): A boolean indicating whether to overwrite an existing
            directory.
    """
    if not isinstance(dir, str) or not dir:
        print(f"ERROR: Bad directory name of `{dir}`.")
        exit()
    elif os.path.isdir(dir) and not overwrite:
        print(f"ERROR: Directory `{dir}` already exists.")
        print("\u2022 Consider deleting the directory or renaming it.")
        exit()
    elif os.path.isdir(dir) and overwrite:
        os.system("rm -r " + dir)
        os.system("mkdir " + dir)
    else:
        os.system("mkdir " + dir)



from mipscripts import utils
import copy
import csv
import os

# Define global variables
global_samples_tsv_header = [
    "sample_name",
    "sample_set",
    "replicate",
    "probe_set",
    "fw",
    "rev",
    "library_prep",
]
global_samples_tsv_header_optional = [
    "capture_plate",
    "quadrant",
    "capture_plate_row",
    "capture_plate_column",
    "384_column",
    "sample_plate",
    "sample_plate_column",
    "sample_plate_row",
    "FW_plate",
    "REV_plate",
    "owner",
]
global_samples_tsv_oldnames = {
    "Library Prep": "library_prep",
    "384 Column": "384_column",
    "row": "sample_plate_row",
    "column": "sample_plate_column",
    "Sample Set": "sample_set",
}


def merge_sampleset(
    set,
    probe,
    sheet,
    mergeon,
    newsheet,
    newfastqdir,
    exclude,
    addcolumn,
    skipfastqwrite,
    renamereplicates,
    collapse,
    ignorereplicateredundancy,
):
    """Flexibly merges sample sets together into one merged sample sheet.

    Merge multiple sample sheets together, selecting the sample sets and probes
    of interest. The user may also create more advanced merges by specifying
    samples matching a pattern or selecting other fields to merge on. In some
    cases, it may be useful to edit the merged sample sheet before combining
    FASTQ files together. To do so, the user may opt to skip the creating of
    FASTQ files and later run `merged_sampleset_write_fastq`.

    Args:
        set (str): The sample sets to aggregate in the merge.
        probe (str): The probe sets to include in the merge.
        sheet (str): The path to the sample sheets that will be merged.
        mergeon (str): The fields to merge on. Fields should be separated by a
            dash.
        newsheet (str): The path to the new merged sample sheet.
        newfastqdir (str): The path to the new FASTQ directory.
        exclude (str): Exclude samples matching a text pattern.
        addcolumn (str): An unexpected column name to be added.
        skipfastqwrite (bool): Whether to skip the writing of new FASTQs. This
            is equivalent to a dry run. By setting this variable to `True`, the
            user has more flexibility in merging FASTQs. The user may use
            `merge_sampleset_write_fastq` following editing of the resulting
            sample sheet.
        renamereplicates (bool): Whether to renumber the replicates based on
            order.
        collapse (bool): Whether to collapse to unique values in columns.
        ignorereplicateredundancy (bool): Whether to collapse multiple
            replicates within a sample sheet.
    """

    # TODO: set default mergedsheet and fastq directory to set_probe
    mergeonfields = mergeon.split("-")
    if addcolumn is None:  # this should be handled by argparse but not!
        addcolumn = []

    # Convert header to snake case
    for f in sheet:
        utils.header_to_snake_case(path=f, overwrite=True)

    total_samples = []
    merged = {}
    # replicate={}
    header = global_samples_tsv_header[:]  # copy, modify
    header_optional = global_samples_tsv_header_optional[
        :
    ]  # copy, don't modify
    headers_to_rename = global_samples_tsv_oldnames  # don't modify

    print(f"CREATING NEW FASTQ DIRECTORY: `{newfastqdir}`")
    utils.make_dir(newfastqdir)
        
    print("PROCESSING SAMPLE SHEETS...")
    for thefile in sheet:
        print(f"LOADING FASTQ DIR {thefile} ...")
        print(header)

        fastqdir = os.path.join(os.path.dirname(thefile), "fastq")
        fastqs = os.listdir(fastqdir)
        fastqs = [f for f in fastqs if any(s in f for s in set)]
        # print ( "...   ",  len(fastqs)    , " fastqs in associated directory" )
        with open(thefile, newline="") as items_file:
            items_reader = csv.DictReader(items_file, delimiter="\t")
            items_total = 0
            items_kept = 0
            items_excluded = 0
            fastqs_kept = 0
            for item in items_reader:
                # rename old names to new names#
                # if items total
                for oldname in headers_to_rename:
                    if oldname in item:
                        item[headers_to_rename[oldname]] = item[oldname]
                        item.pop(oldname)
                        if items_total == 0:
                            print(
                                "... ... RENAMED old name '{}' to '{}'"
                            ).format(oldname, headers_to_rename[oldname])
                if items_total == 0:
                    # process the headerline to see what headers are present
                    # determine if we need to add another line to current header.
                    # print (item)
                    for name in item:
                        if name not in header:
                            if name in header_optional or name in addcolumn:
                                print("... ... NOTE: ", name, " added to data ")
                                header.append(name)
                            else:
                                print("ERROR: Improper column name.")
                                print(
                                    f"\u2022 '{name}' is not a proper column name."
                                )
                                print("\u2022 Use '--addcolumn' to override.")
                                exit(1)

                items_total += 1
                if not any(s == item["sample_set"] for s in set):
                    continue
                if not any(p in item["probe_set"] for p in probe):
                    continue
                # keep item with proper probeset and sampleset
                items_kept += 1

                startfastqname = "{}-{}-{}_".format(
                    item["sample_name"], item["sample_set"], item["replicate"]
                )
                item["samplelist"] = thefile
                item["replicate_old"] = item["replicate"]
                item["fastq"] = [
                    fastqdir + "/" + i
                    for i in fastqs
                    if i.startswith(startfastqname)
                ]
                item["fastq"].sort()
                fastqs_kept += len(item["fastq"])
                item["mergeon"] = []
                for f in mergeonfields:
                    item["mergeon"].append(item[f])
                item["mergeon"] = "-".join(item["mergeon"])
                if exclude and any(ele in item["mergeon"] for ele in exclude):
                    items_excluded += 1
                    continue
                # print (startfastqname)
                # find the fastqs
                total_samples.append(item)
                merged[item["mergeon"]] = 0
                # replicate[item["sample_name"]+"-"+ item["sample_set"]]=1
            print("    ", items_excluded, " excluded samples")
            print(
                "   ",
                items_kept,
                " of ",
                items_total,
                " total with ",
                fastqs_kept,
                "fastqs kept",
            )
    # print ("HEADER",header)
    # print (EXtotal_samples[0:1])
    print(
        "Total samples ",
        len(total_samples),
        " merging into ",
        len(merged),
        " samples",
    )
    total_samples.sort(key=lambda x: x["mergeon"])

    print("############################")
    print("MERGE ALL THE SAMPLE SHEET ROWS ")
    # merge the list and copy the files ####################
    replicate = {}
    merged_samples = []
    count = 0
    header = header + [
        "fastq",
        "samplelist",
        "mergeon",
        "replicate_old",
    ]  # add on additional headers to report
    for mkey in sorted(merged):
        mergeset = [i for i in total_samples if i["mergeon"] == mkey]
        count += 1
        # print (count, mkey, "records:",len(mergeset))
        mergeditem = None
        name = None

        # Create a merge record from first one and add the others
        for item in mergeset:
            if mergeditem is None:
                mergeditem = copy.deepcopy(item)
                # make sure merged item has all the potential names from header
                # set file name and replicated based on new name and sampleset#
                for f in header:
                    if f in mergeditem:
                        mergeditem[f] = [mergeditem[f]]
                    else:
                        mergeditem[f] = ["NA"]
            else:
                # print ("MERGEDITEM", mergeditem)
                for f in header:
                    if f in item:
                        mergeditem[f].append(item[f])
                    else:
                        mergeditem[f].append("NA")

        # Collapse in mergeitem sampleset, name and mergeon and error if not the same
        for f in ["sample_name", "sample_set", "mergeon"]:
            mergeditem[f] = list(set(mergeditem[f]))
            if len(mergeditem[f]) == 1:
                mergeditem[f] = mergeditem[f][0]
            else:
                print(
                    "!ERROR! ", mergeditem[f], "for", f, " is not consistent!"
                )
                exit(1)

        # Set the replicate
        name = mergeditem["sample_name"] + "-" + mergeditem["sample_set"]
        mergeditem["replicate_old"] = mergeditem["replicate"]
        mergeditem["replicate"] = mergeditem["replicate"][0]
        if renamereplicates:
            # rename replicates based on name-set
            if name in replicate:
                replicate[name] += 1
            else:
                replicate[name] = 1
            mergeditem["replicate"] = replicate[name]
        else:
            # replicate name based on name-set-replicate exists than error
            name += "-" + mergeditem["replicate"]
            if name in replicate:
                print("!ERROR!", name, ") already exists!")
                exit(1)
            else:
                replicate[name] = 1

        # Collapse probesets
        mergeditem["probe_set"] = ",".join(mergeditem["probe_set"]).split(",")
        mergeditem["probe_set"] = list(set(mergeditem["probe_set"]))
        mergeditem["probe_set"].sort()
        mergeditem["probe_set"] = ",".join(mergeditem["probe_set"])
        merged_samples.append(mergeditem)
    print("################")
    print("MERGE THE FASTQS...")
    for mergeditem in merged_samples:
        # Merge the fastqs
        # Note that this is slow
        fastqname = "{}-{}-{}".format(
            mergeditem["sample_name"],
            mergeditem["sample_set"],
            mergeditem["replicate"],
        )
        writeoperator = " > "  # initial write
        for pair in mergeditem["fastq"]:
            if len(pair) == 0:
                # print ("emptty value")
                continue
            elif (len(pair) % 2) == 0:  # 2,4,6,8
                # expectation is each replicate record has pair of fastqs (R1 & R2)
                # potential to have more than one replicate in a sequencing run but that is weird
                # print (pair)
                if len(pair) > 2:
                    if ignorereplicateredundancy:
                        print(
                            "WARN: (",
                            mergeditem["mergeon"],
                            ") just taking one pair of fastqs -- likely duplicate or bad replicate numbers!",
                        )
                    else:
                        print(mergeditem)
                        print(
                            "!ERROR! More than just a single pair for replicate!",
                            pair,
                            "\nThis could be due to not properly numbering replicates!",
                            "\nOr multiple demultiplexing runs into same directory?",
                            "\nOr because you meant to...",
                            "\nTo override use --ignorereplicateredundancy!",
                        )
                        badcount = len(
                            [m for m in total_samples if len(m["fastq"]) > 2]
                        )
                        print("NOTE: ", badcount, "have more than  2 fastqs!")
                        print(
                            "NOTE: --ignorereplicateredundancy tosses the extra fastqs"
                        )
                        exit(1)

                if not skipfastqwrite:  # this skips lengthy process of writing
                    print(
                        "...",
                        mergeditem["mergeon"],
                        "...",
                        pair[0][-65:],
                        "...",
                    )
                    os.system(
                        "cat  "
                        + pair[0]
                        + writeoperator
                        + newfastqdir
                        + "/"
                        + fastqname
                        + "_R1_001.fastq.gz"
                    )
                    os.system(
                        "cat  "
                        + pair[1]
                        + writeoperator
                        + newfastqdir
                        + "/"
                        + fastqname
                        + "_R2_001.fastq.gz"
                    )
                    writeoperator = " >> "  # now appending after initial files
                # else:
                #  for i in range (0, len(pair),2):
                #     print (pair[i],"\n   ", pair[i+1])
            else:
                print("ODD PAIR ERROR ", pair)
                print(" could a R1 or R2 fastq file been deleted???")
                exit(1)
            writeoperator = " >> "  # now appending
        # Make fields non-redundant (always make probeset)
        mergeditem["fastq"] = [
            fq for sublist in mergeditem["fastq"] for fq in sublist
        ]
        # print (mergeditem)
    # if collapse :
    #   uniquefields=fieldstocollapse
    #  for f in uniquefields :
    #  if f in mergeditem:
    #      #print (f, mergeditem[f])
    #     mergeditem[f]= list(set(mergeditem[f]))
    # for f in fieldstocollapse :
    #  if f in mergeditem:
    #    mergeditem[f]= ",".join(mergeditem[f])

    # print ("NEW COPY ", mergeditem)
    # write to a file ################################
    # write header and additional data from merge
    print("WRITING TSV: ", newsheet)
    with open(newsheet, "wt") as out_file:
        tsv_writer = csv.writer(out_file, delimiter="\t")
        tsv_writer.writerow(header)
        for item in merged_samples:
            row = []
            for f in header:
                if type(item[f]) is list:
                    if collapse:
                        item[f] = list(set(item[f]))
                    item[f] = ",".join(item[f])
                row.append(item[f])
            tsv_writer.writerow(row)
